//-----------------------------------------------------------------------------
// Copyright  2001 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//-----------------------------------------------------------------------------
// package	vrb
// fullname	virtual ring buffer
// homepage	http://phil.ipal.org/freeware/vrb/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/poll.h>
#include <unistd.h>

#include <vrb.h>

#ifndef EAGAIN
#define EAGAIN EWOULDBLOCK
#endif


//-----------------------------------------------------------------------------
// program	iobuffer
//
// purpose	Demonstrate and test the vrb library functions by
//		implementing a simple pipe buffering command.
//
// syntax	buffer [options]
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//-----------------------------------------------------------------------------
int
main (
    int		argc
    ,
    char * *	argv
    ,
    char * *	envp
    )
{
    struct pollfd	poll_list	[2]	;

    vrb			io_buf		;

    unsigned long long	total_read	;
    unsigned long long	total_write	;

    size_t		page_size	;
    size_t		buf_size	;
    size_t		read_size	;
    size_t		write_size	;

    int			block		;
    int			die		;
    int			eof		;
    int			error_count	;
    int			fd		;
    int			idled		;
    int			ready		;
    int			timeout		;


    error_count = 0;
    total_read = 0;
    total_write = 0;
    timeout = 5000;

    page_size = sysconf( _SC_PAGESIZE );
    read_size = page_size;
    write_size = page_size;
    buf_size = 65536;
    if ( buf_size < page_size ) buf_size = page_size;

    -- argc;
    ++ argv;
    while ( argc && * argv && * * argv == '-' ) {

	if ( strcmp( * argv, "-b" ) == 0 ) {
	    -- argc;
	    ++ argv;
	    if ( argc && * argv ) {
		buf_size = strtoul( * argv, NULL, 0 );
	    }
	}

	else if ( strcmp( * argv, "-r" ) == 0 ) {
	    -- argc;
	    ++ argv;
	    if ( argc && * argv ) {
		read_size = strtoul( * argv, NULL, 0 );
	    }
	}

	else if ( strcmp( * argv, "-t" ) == 0 ) {
	    -- argc;
	    ++ argv;
	    if ( argc && * argv ) {
		timeout = strtoul( * argv, NULL, 0 );
	    }
	}

	else if ( strcmp( * argv, "-w" ) == 0 ) {
	    -- argc;
	    ++ argv;
	    if ( argc && * argv ) {
		write_size = strtoul( * argv, NULL, 0 );
	    }
	}

	else {
	    fprintf( stderr, "unknown option: %s\n", * argv );
	    ++ error_count;
	}

	-- argc;
	++ argv;
    }

    if ( error_count > 0 ) return 1;

    io_buf = vrb_new( buf_size, NULL );
    if ( ! io_buf ) {
	fprintf( stderr, "error creating I/O buffer: %s\n", strerror( errno ) );
	return 1;
    }

    for ( fd = 0; fd <= 1; ++ fd ) {
	if ( fcntl( fd, F_SETFL, O_NONBLOCK ) < 0 ) {
	    fprintf( stderr, "error setting O_NONBLOCK on file descriptor %d: %s\n", fd, strerror( errno ) );
	    return 1;
	}
    }

    die = 0;
    eof = 0;

    while ( ! die ) {

	idled = 0;
	block = 0;
	ready = 0;

	if ( eof == 0 ) {
	    poll_list[0].fd = -1;
	    poll_list[0].events = 0;
	    if ( vrb_space_len( io_buf ) == 0 ) {
		++ idled;
	    } else {
		size_t result;

		result = vrb_read( io_buf, 0, read_size );
		if ( result == ~0 ) {
		    if ( errno == 0 ) {
			++ eof;
		    }
		    else if ( errno == EAGAIN ) {
			++ block;
			poll_list[0].fd = 0;
			poll_list[0].events = POLLIN;
		    } else {
			fprintf( stderr, "read  unexpected error: %s\n", strerror( errno ) );
			++ eof;
		    }
		} else {
		    ++ ready; // we may be able to read more in next pass
		    total_read += result;
		}
	    }
	}

	poll_list[1].fd = -2;
	poll_list[1].events = 0;
	if ( vrb_data_len( io_buf ) == 0 ) {
	    ++ idled;
	    if ( eof ) {
		break;
	    }
	} else {
	    size_t result;

	    result = vrb_write( io_buf, 1, write_size );
	    if ( result == ~0 ) {
		if ( errno == EAGAIN ) {
		    ++ block;
		    poll_list[1].fd = 1;
		    poll_list[1].events = POLLOUT;
		} else {
		    fprintf( stderr, "write unexpected error: %s\n", strerror( errno ) );
		    die = 1;
		}
	    } else {
		++ ready; // we may be able to write more in next pass
		total_write += result;
	    }
	}

	if ( idled + block + ready + eof != 2 ) {
	    fprintf( stderr,
		     "idled = %d    block = %d    ready = %d    eof = %d\n",
		     idled,block,ready,eof );
	    return 1;
	}

	if ( ready == 0 ) {
	    int n;

	    if ( block == 0 ) {
		fprintf( stderr, "DEADLOCK: nothing ready, nothing blocking, not good\n" );
		return 1;
	    }

	    poll_list[0].revents = 0;
	    poll_list[1].revents = 0;

	    n = poll( poll_list, 2, timeout );

	}

    }

    vrb_destroy( io_buf );

    return die;
}
