//-----------------------------------------------------------------------------
// Copyright  2001 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
//-----------------------------------------------------------------------------
// package	vrb
// fullname	virtual ring buffer
// homepage	http://phil.ipal.org/freeware/vrb/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	vrb@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// Thanks to Kasper Dupont, John Reiser, and Linus Torvalds for the help
// and various ideas on how to interface with the operating system.
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/shm.h>
#include <unistd.h>

#include "vrb_lib.h"

//-----------------------------------------------------------------------------
// MAP_ANON probably is defined but if not, maybe MAP_ANONYMOUS is.
//-----------------------------------------------------------------------------
#ifndef MAP_ANON
#define MAP_ANON MAP_ANONYMOUS
#endif

__MACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	vrb_new
//
// purpose	Make a new instance of a vrb, returning the pointer to the
//		vrb structure.
//
//		If no name is given, use SysV shared memory for the shared
//		memory object, and swap for backing store.
//
//		If a file name is given, use POSIX memory mapping into that
//		file for the shared memory object, and that file for backing
//		store.
//
// arguments	1 (size_t) minimum buffer space requested
//		2 (const char *) filename to memory map into
//
// returns	(vrb) pointer to vrb
//		(vrb) NULL if error (see errno)
//-----------------------------------------------------------------------------
#define vrb_new(s,n) (vrb_init(((vrb)(NULL)),(s),(n)))
__MACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	vrb_init
//
// purpose	Initialize a vrb with an empty space of at least the size
//		requested.  The actual size will be rounded up to meet system
//		requirements, if needed.
//		
//		If no vrb is given, create a new one.
//
//		If no name is given, use SysV shared memory for the shared
//		memory object, and swap for backing store.
//
//		If a file name is given, use POSIX memory mapping into that
//		file for the shared memory object, and that file for backing
//		store.
//
// arguments	1 (vrb) NULL or an existing vrb to initialize
//		2 (size_t) size requested, which will be rounded up as needed
//		3 (const char *) filename to memory map into
//
// returns	(vrb) pointer to initialize vrb
//		(vrb) NULL if there was an error
//-----------------------------------------------------------------------------
vrb
vrb_init (
    vrb			arg_vrb
    ,
    size_t		arg_size
    ,
    const char *	arg_name
    )
__PROTO_END__
{
    char *	lower_ptr	;
    char *	upper_ptr	;

    char *	mmap_addr	;
    char *	shmat_addr	;

    vrb		vrb_ptr		;

    size_t	req_size	;

    int		fd		;
    int		shm_id		;

    int		do_free		;
    int		do_munmap	;
    int		do_shmdt	;

    char *	tempname	;


    //--------------------------------------------------------------
    // Initialize cleanup flags to indicate nothing yet to clean up.
    //--------------------------------------------------------------
    fd			= -1;
    shm_id		= -1;
    shmat_addr		= (char *) -1;
    mmap_addr		= (char *) MAP_FAILED;

    do_free		= 0;
    do_munmap		= 0;
    do_shmdt		= 0;

    tempname		= NULL;

    //-----------------------------------------------------------------
    // Make sure twice the requested size can be represented in size_t.
    // Let the system impose smaller sizes on its own.
    //-----------------------------------------------------------------
    if ( arg_size > ( ( ~ ( (size_t) 0 ) ) >> 2 ) ) {
	errno = EINVAL;
	return NULL;
    }

    //--------------------------------------------------
    // Make sure we have a vrb provided or allocate one.
    //--------------------------------------------------
    if ( arg_vrb ) {
	vrb_ptr = arg_vrb;
    } else {
	if ( ! ( vrb_ptr = (vrb) malloc( sizeof (struct vrb) ) ) ) {
	    errno = ENOMEM;
	    return NULL;
	}
	do_free = 1;
    }

    //-------------------------------------------------------------------------
    // Get the requested size rounded up to a multiple of the system page size.
    //-------------------------------------------------------------------------
#ifdef _SC_PAGESIZE
    req_size = sysconf( _SC_PAGESIZE );
#else
    req_size = getpagesize();
#endif
    req_size += ( arg_size - 1 ) - ( ( arg_size - 1 ) % req_size );

    //---------------------------------------
    // Start initializing the new vrb struct.
    //---------------------------------------
    vrb_ptr->buf_size		= req_size;
    vrb_ptr->flags		= 0;
    
    //------------------------------------------------------------------
    // In order to place 2 virtual memory segments at adjacent locations
    // use an anonymous mmap() call to find some space twice as large.
    //------------------------------------------------------------------
    lower_ptr = (char *) mmap( 0,
			       req_size + req_size,
			       PROT_READ | PROT_WRITE,
			       MAP_ANON | MAP_SHARED,
			       -1,
			       0 );
    upper_ptr = lower_ptr;
    if ( lower_ptr == (char *) MAP_FAILED ) {
	goto error_cleanup;
    }
    upper_ptr = lower_ptr + req_size;
    do_munmap = 1;

#ifdef VRB_ENV_TEMPNAME
    //--------------------------------------------------
    // If no explicit file name is given as an argument,
    // then check the environment for a name.
    // Check the environment only if NOT root.
    //--------------------------------------------------
    if ( ! arg_name && geteuid() != 0 ) {
	arg_name = getenv( VRB_ENV_TEMPNAME );
    }
#endif

    //----------------------------------------------------
    // If still no file name, then use SysV shared memory.
    //----------------------------------------------------
    if ( ! arg_name ) {

	//-- Unmap to allow shmat() to take the reserved space.
	if ( 0 > munmap( lower_ptr, req_size + req_size ) ) {
	    goto error_cleanup;
	}
	do_munmap = 0;

	//-- Create a private shared memory segment.
	shm_id = shmget( IPC_PRIVATE, req_size, IPC_CREAT | 0700 );
	if ( 0 > shm_id ) {
	    goto error_cleanup;
	}

	//-- Attach the shared segment at the lower buffer.
	shmat_addr = shmat( shm_id, lower_ptr, 0 );
	if ( shmat_addr != lower_ptr ) {
	    goto error_cleanup;
	}
	do_shmdt = 1;

	//-- Attach the shared segment at the upper buffer.
	shmat_addr = shmat( shm_id, upper_ptr, 0 );
	if ( shmat_addr != upper_ptr ) {
	    goto error_cleanup;
	}
	do_shmdt = 2;
	shmat_addr = (char *) -1;

	//-- Mark the segment to be deleted after last detach.
	if ( 0 > shmctl( shm_id, IPC_RMID, NULL ) ) {
	    goto error_cleanup;
	}
	shm_id = -1;

	//-- Remember this vrb uses SysV shared memory.
	vrb_set_shmat( vrb_ptr );

    }

    //-------------------------------------------------------
    // Else use POSIX memory mapping with the specified name.
    //-------------------------------------------------------
    else {

	//-- Allocate temporary space for mkstemp() and copy the name to it.
	if ( ! ( tempname = alloca( strlen( arg_name ) + 1 ) ) ) {
	    goto error_cleanup;
	}
	strcpy( tempname, arg_name );

	//-- Open a temporary file, unique if possible.
	if ( 0 > ( fd = mkstemp( tempname ) ) &&
	     ( errno != EINVAL ||
	       0 > ( fd = open( tempname, O_RDWR | O_EXCL, 0600 ) ) ) ) {
	    goto error_cleanup;
	}
		
	//-- Unlink the temporary file immediately now that it is open.
	if ( 0 > unlink( tempname ) ) {
	    goto error_cleanup;
	}

	//-- Stretch the file out to the size we need.
	if ( 0 > ( (long) lseek( fd, req_size, SEEK_CUR ) ) ) {
	    goto error_cleanup;
	}

	//-- Map over the lower space.
	mmap_addr = mmap( lower_ptr,
			  req_size,
			  PROT_READ | PROT_WRITE,
			  MAP_FIXED | MAP_SHARED,
			  fd,
			  0 );
	if ( mmap_addr != lower_ptr ) {
	    goto error_cleanup;
	}

	//-- Map over the upper space.
	mmap_addr = mmap( upper_ptr,
			  req_size,
			  PROT_READ | PROT_WRITE,
			  MAP_FIXED | MAP_SHARED,
			  fd,
			  0 );
	if ( mmap_addr != upper_ptr ) {
	    goto error_cleanup;
	}

	//-- Close the file now that its space is mapped.
	close( fd );
	fd = -1;

	//-- Remember this vrb uses POSIX memory mapping.
	vrb_set_mmap( vrb_ptr );

    }

    //--------------------------------------------
    // Fill in and return the vrb header struct.
    //--------------------------------------------
    vrb_ptr->lower_ptr	= (char *) lower_ptr;
    vrb_ptr->upper_ptr	= (char *) upper_ptr;
    vrb_ptr->first_ptr	= (char *) lower_ptr;
    vrb_ptr->last_ptr	= (char *) lower_ptr;

    return vrb_ptr;


//--------------------------------------------------------------
// Do all the failure clean up here, then return an error value.
//--------------------------------------------------------------
 error_cleanup:
    {
	int keep_errno;

	keep_errno = errno;

	if ( do_shmdt > 0 ) {
	    if ( do_shmdt > 1 ) shmdt( upper_ptr );
	    shmdt( lower_ptr );
	}

	if ( shmat_addr != (char *) -1 ) {
	    shmdt( shmat_addr );
	}

	if ( shm_id >= 0 ) {
	    shmctl( shm_id, IPC_RMID, NULL );
	}

	if ( fd >= 0 ) {
	    close( fd );
	}

	if ( mmap_addr != MAP_FAILED ) {
	    munmap( mmap_addr, req_size );
	}

	if ( do_munmap ) {
	    munmap( lower_ptr, req_size + req_size );
	}

	if ( do_free ) {
	    free( vrb_ptr );
	}

	errno = keep_errno;
    }

    return NULL;
}
